@@ -90,12 +90,6 @@
 GST_DEBUG_CATEGORY_STATIC (pipeline_debug);
 #define GST_CAT_DEFAULT pipeline_debug
 
-static const GstElementDetails gst_pipeline_details =
-GST_ELEMENT_DETAILS ("Pipeline object",
-    "Generic/Bin",
-    "Complete pipeline object",
-    "Erik Walthinsen <omega@cse.ogi.edu>, Wim Taymans <wim@fluendo.com>");
-
 /* Pipeline signals and args */
 enum
 {
@@ -181,7 +175,10 @@ gst_pipeline_base_init (gpointer g_class)
 {
   GstElementClass *gstelement_class = GST_ELEMENT_CLASS (g_class);
 
-  gst_element_class_set_details (gstelement_class, &gst_pipeline_details);
+  gst_element_class_set_details_simple (gstelement_class, "Pipeline object",
+      "Generic/Bin",
+      "Complete pipeline object",
+      "Erik Walthinsen <omega@cse.ogi.edu>, Wim Taymans <wim@fluendo.com>");
 }
 
 static void
@@ -212,7 +209,7 @@ gst_pipeline_class_init (gpointer g_class, gpointer class_data)
       g_param_spec_uint64 ("delay", "Delay",
           "Expected delay needed for elements "
           "to spin up to PLAYING in nanoseconds", 0, G_MAXUINT64, DEFAULT_DELAY,
-          G_PARAM_READWRITE));
+          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
   /**
    * GstPipeline:auto-flush-bus:
@@ -227,7 +224,7 @@ gst_pipeline_class_init (gpointer g_class, gpointer class_data)
       g_param_spec_boolean ("auto-flush-bus", "Auto Flush Bus",
           "Whether to automatically flush the pipeline's bus when going "
           "from READY into NULL state", DEFAULT_AUTO_FLUSH_BUS,
-          G_PARAM_READWRITE));
+          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
   gobject_class->dispose = GST_DEBUG_FUNCPTR (gst_pipeline_dispose);
 
@@ -371,11 +368,8 @@ gst_pipeline_change_state (GstElement * element, GstStateChange transition)
     case GST_STATE_CHANGE_PAUSED_TO_PLAYING:
     {
       GstClockTime new_base_time;
-      GstQuery *query;
-      GstClockTime min_latency, max_latency;
       GstClockTime start_time, stream_time, delay;
       gboolean new_clock, update_stream_time, update_clock;
-      gboolean res;
       GstClock *cur_clock;
 
       GST_DEBUG_OBJECT (element, "selecting clock and base_time");
@@ -457,40 +451,6 @@ gst_pipeline_change_state (GstElement * element, GstStateChange transition)
 
       if (cur_clock)
         gst_object_unref (cur_clock);
-
-      /* determine latency in this pipeline */
-      GST_DEBUG_OBJECT (element, "querying pipeline latency");
-      query = gst_query_new_latency ();
-      if (gst_element_query (element, query)) {
-        gboolean live;
-
-        gst_query_parse_latency (query, &live, &min_latency, &max_latency);
-
-        GST_DEBUG_OBJECT (element,
-            "configuring min latency %" GST_TIME_FORMAT ", max latency %"
-            GST_TIME_FORMAT ", live %d", GST_TIME_ARGS (min_latency),
-            GST_TIME_ARGS (max_latency), live);
-
-        /* configure latency on elements */
-        res =
-            gst_element_send_event (element,
-            gst_event_new_latency (min_latency));
-        if (res) {
-          GST_INFO_OBJECT (element, "configured latency of %" GST_TIME_FORMAT,
-              GST_TIME_ARGS (min_latency));
-        } else {
-          GST_WARNING_OBJECT (element,
-              "failed to configure latency of %" GST_TIME_FORMAT,
-              GST_TIME_ARGS (min_latency));
-          GST_ELEMENT_WARNING (element, CORE, CLOCK, (NULL),
-              ("Failed to configure latency of %" GST_TIME_FORMAT,
-                  GST_TIME_ARGS (min_latency)));
-        }
-      } else {
-        /* this is not a real problem, we just don't configure any latency. */
-        GST_WARNING_OBJECT (element, "failed to query pipeline latency");
-      }
-      gst_query_unref (query);
       break;
     }
     case GST_STATE_CHANGE_PLAYING_TO_PAUSED:
@@ -603,7 +563,7 @@ gst_pipeline_handle_message (GstBin * bin, GstMessage * message)
  * gst_pipeline_get_bus:
  * @pipeline: a #GstPipeline
  *
- * Gets the #GstBus of @pipeline.
+ * Gets the #GstBus of @pipeline. The bus allows applications to receive #GstMessages.
  *
  * Returns: a #GstBus, unref after usage.
  *
