@@ -45,7 +45,7 @@ n_print (const char *format, ...)
   gint retval;
 
   if (_name)
-    g_print (_name);
+    g_print ("%s", _name);
 
   va_start (args, format);
   retval = g_vprintf (format, args);
@@ -122,7 +122,6 @@ print_query_types (const GstQueryType * types)
   }
 }
 
-#ifndef GST_DISABLE_ENUMTYPES
 #if 0
 static void
 print_event_masks (const GstEventMask * masks)
@@ -172,12 +171,6 @@ print_event_masks (const GstEventMask * masks)
   }
 }
 #endif
-#else
-static void
-print_event_masks (const GstEventMask * masks)
-{
-}
-#endif
 
 static char *
 get_rank_name (gint rank)
@@ -225,7 +218,7 @@ print_hierarchy (GType type, gint level, gint * maxlevel)
     print_hierarchy (parent, level, maxlevel);
 
   if (_name)
-    g_print (_name);
+    g_print ("%s", _name);
 
   for (i = 1; i < *maxlevel - level; i++)
     g_print ("      ");
@@ -247,17 +240,17 @@ print_interfaces (GType type)
   if (ifaces) {
     if (n_ifaces) {
       if (_name)
-        g_print (_name);
+        g_print ("%s", _name);
       g_print (_("Implemented Interfaces:\n"));
       iface = ifaces;
       while (*iface) {
         if (_name)
-          g_print (_name);
+          g_print ("%s", _name);
         g_print ("  %s\n", g_type_name (*iface));
         iface++;
       }
       if (_name)
-        g_print (_name);
+        g_print ("%s", _name);
       g_print ("\n");
     }
     g_free (ifaces);
@@ -474,7 +467,7 @@ print_element_properties_info (GstElement * element)
           while (values[j].value_name) {
             g_print ("\n");
             if (_name)
-              g_print (_name);
+              g_print ("%s", _name);
             g_print ("%-23.23s    (%d): %-16s - %s", "",
                 values[j].value, values[j].value_nick, values[j].value_name);
             j++;
@@ -520,7 +513,7 @@ print_element_properties_info (GstElement * element)
           while (values[j].value_name) {
             g_print ("\n");
             if (_name)
-              g_print (_name);
+              g_print ("%s", _name);
             g_print ("%-23.23s    (0x%08x): %-16s - %s", "",
                 values[j].value, values[j].value_nick, values[j].value_name);
             j++;
@@ -544,7 +537,14 @@ print_element_properties_info (GstElement * element)
             n_print ("%-23.23s Pointer.", "");
           }
         } else if (param->value_type == G_TYPE_VALUE_ARRAY) {
-          n_print ("%-23.23s Array of GValues", "");
+          GParamSpecValueArray *pvarray = G_PARAM_SPEC_VALUE_ARRAY (param);
+
+          if (pvarray->element_spec) {
+            n_print ("%-23.23s Array of GValues of type \"%s\"", "",
+                g_type_name (pvarray->element_spec->value_type));
+          } else {
+            n_print ("%-23.23s Array of GValues", "");
+          }
         } else if (GST_IS_PARAM_SPEC_FRACTION (param)) {
           GstParamSpecFraction *pfraction = GST_PARAM_SPEC_FRACTION (param);
 
@@ -559,6 +559,9 @@ print_element_properties_info (GstElement * element)
                 gst_value_get_fraction_numerator (&value),
                 gst_value_get_fraction_denominator (&value));
 
+        } else if (GST_IS_PARAM_SPEC_MINI_OBJECT (param)) {
+          n_print ("%-23.23s MiniObject of type \"%s\"", "",
+              g_type_name (param->value_type));
         } else {
           n_print ("%-23.23s Unknown type %ld \"%s\"", "", param->value_type,
               g_type_name (param->value_type));
@@ -694,7 +697,6 @@ print_clocking_info (GstElement * element)
   }
 }
 
-#ifndef GST_DISABLE_INDEX
 static void
 print_index_info (GstElement * element)
 {
@@ -707,12 +709,39 @@ print_index_info (GstElement * element)
     n_print ("Element has no indexing capabilities.\n");
   }
 }
-#else
+
 static void
-print_index_info (GstElement * element)
+print_uri_handler_info (GstElement * element)
 {
+  if (GST_IS_URI_HANDLER (element)) {
+    const gchar *uri_type;
+    gchar **uri_protocols;
+
+    if (gst_uri_handler_get_uri_type (GST_URI_HANDLER (element)) == GST_URI_SRC)
+      uri_type = "source";
+    else if (gst_uri_handler_get_uri_type (GST_URI_HANDLER (element)) ==
+        GST_URI_SINK)
+      uri_type = "sink";
+    else
+      uri_type = "unknown";
+
+    uri_protocols = gst_uri_handler_get_protocols (GST_URI_HANDLER (element));
+
+    n_print ("\n");
+    n_print ("URI handling capabilities:\n");
+    n_print ("  Element can act as %s.\n", uri_type);
+
+    if (uri_protocols && *uri_protocols) {
+      n_print ("  Supported URI protocols:\n");
+      for (; *uri_protocols != NULL; uri_protocols++)
+        n_print ("    %s\n", *uri_protocols);
+    } else {
+      n_print ("  No supported URI protocols\n");
+    }
+  } else {
+    n_print ("Element has no URI handling capabilities.\n");
+  }
 }
-#endif
 
 static void
 print_pad_info (GstElement * element)
@@ -855,7 +884,7 @@ print_signal_info (GstElement * element)
 
       for (j = 0; j < query->n_params; j++) {
         if (_name)
-          g_print (_name);
+          g_print ("%s", _name);
         if (G_TYPE_IS_FUNDAMENTAL (query->param_types[j])) {
           g_print (",\n%s%s arg%d", indent,
               g_type_name (query->param_types[j]), j);
@@ -870,7 +899,7 @@ print_signal_info (GstElement * element)
 
       if (k == 0) {
         if (_name)
-          g_print (_name);
+          g_print ("%s", _name);
         g_print (",\n%sgpointer user_data);\n", indent);
       } else
         g_print (");\n");
@@ -939,18 +968,14 @@ print_element_list (gboolean print_all)
           g_print ("%s:  %s: %s\n", plugin->desc.name,
               GST_PLUGIN_FEATURE_NAME (factory),
               gst_element_factory_get_longname (factory));
-      }
-#ifndef GST_DISABLE_INDEX
-      else if (GST_IS_INDEX_FACTORY (feature)) {
+      } else if (GST_IS_INDEX_FACTORY (feature)) {
         GstIndexFactory *factory;
 
         factory = GST_INDEX_FACTORY (feature);
         if (!print_all)
           g_print ("%s:  %s: %s\n", plugin->desc.name,
               GST_PLUGIN_FEATURE_NAME (factory), factory->longdesc);
-      }
-#endif
-      else if (GST_IS_TYPE_FIND_FACTORY (feature)) {
+      } else if (GST_IS_TYPE_FIND_FACTORY (feature)) {
         GstTypeFindFactory *factory;
 
         factory = GST_TYPE_FIND_FACTORY (feature);
@@ -1036,17 +1061,13 @@ print_plugin_features (GstPlugin * plugin)
       n_print ("  %s: %s\n", GST_PLUGIN_FEATURE_NAME (factory),
           gst_element_factory_get_longname (factory));
       num_elements++;
-    }
-#ifndef GST_DISABLE_INDEX
-    else if (GST_IS_INDEX_FACTORY (feature)) {
+    } else if (GST_IS_INDEX_FACTORY (feature)) {
       GstIndexFactory *factory;
 
       factory = GST_INDEX_FACTORY (feature);
       n_print ("  %s: %s\n", GST_OBJECT_NAME (factory), factory->longdesc);
       num_indexes++;
-    }
-#endif
-    else if (GST_IS_TYPE_FIND_FACTORY (feature)) {
+    } else if (GST_IS_TYPE_FIND_FACTORY (feature)) {
       GstTypeFindFactory *factory;
 
       factory = GST_TYPE_FIND_FACTORY (feature);
@@ -1093,14 +1114,12 @@ print_element_features (const gchar * element_name)
   GstPluginFeature *feature;
 
   /* FIXME implement other pretty print function for these */
-#ifndef GST_DISABLE_INDEX
   feature = gst_default_registry_find_feature (element_name,
       GST_TYPE_INDEX_FACTORY);
   if (feature) {
     n_print ("%s: an index\n", element_name);
     return 0;
   }
-#endif
   feature = gst_default_registry_find_feature (element_name,
       GST_TYPE_TYPE_FIND_FACTORY);
   if (feature) {
@@ -1156,6 +1175,7 @@ print_element_info (GstElementFactory * factory, gboolean print_names)
   print_implementation_info (element);
   print_clocking_info (element);
   print_index_info (element);
+  print_uri_handler_info (element);
   print_pad_info (element);
   print_element_properties_info (element);
   print_signal_info (element);
